@@ -2,6 +2,7 @@
 
 from __future__ import absolute_import, division, print_function, unicode_literals
 
+
 unicode = type("")
 
 class BytesOnly(bytes):
@@ -22,3 +23,13 @@ class UnicodeOnly(unicode):
 
     def __ne__(self, o):
         return not self == o
+
+try:
+    class LongOnly(long):  # Python 2
+        def __eq__(self, o):
+            return isinstance(o, long) and long.__cmp__(self, o) == 0  # noqa
+
+        def __ne__(self, o):
+            return not self == o
+except NameError:          # Python 3
+    LongOnly = int
